/* 
 * Copyright (c) 2023 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 *
 * This file is part of SPD.
 *
 * SPD is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * SPD is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SPD. If not, see <https://www.gnu.org/licenses/>.
 *
 * This file incorporates work covered by the following copyright and permission
 * notice:
 *
 *     Copyright 2017 Douman
 *
 *     Licensed under the Apache License, Version 2.0 (the "License");
 *     you may not use this file except in compliance with the License.
 *     You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */

extern crate alloc;
use alloc::string::String;

pub fn to_hyphenated_lower_case(input: &str) -> String {
    let mut output = String::with_capacity(input.len());

    let mut chars = input.chars();
    let mut prev_upper = false;
    if let Some(ch) = chars.next() {
        if ch.is_uppercase() {
            prev_upper = true;
            for ch in ch.to_lowercase() {
                output.push(ch);
            }
        } else {
            output.push(ch);
        }
    }

    for ch in chars {
        if ch.is_uppercase() {
            if !prev_upper {
                output.push('-');
            }

            for ch in ch.to_lowercase() {
                output.push(ch);
            }

            prev_upper = true;
        } else {
            output.push(ch);

            prev_upper = false;
        }
    }

    output
}
