/* 
 * Copyright (c) 2023 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: LGPL-3.0-or-later
 *
 * This file is part of SPD.
 *
 * SPD is free software: you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option) any
 * later version.
 *
 * SPD is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with SPD. If not, see <https://www.gnu.org/licenses/>.
 *
 * This file incorporates work covered by the following copyright and permission
 * notice:
 *
 *     Copyright 2017 Douman
 *
 *     Licensed under the Apache License, Version 2.0 (the "License");
 *     you may not use this file except in compliance with the License.
 *     You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 */

///Simple split of string into arguments
pub struct Split<'a> {
    string: &'a str,
}

impl<'a> Split<'a> {
    ///Creates new instance
    pub const fn from_str(string: &'a str) -> Self {
        Self {
            string
        }
    }

    #[inline(always)]
    ///Retrieves next argument
    pub fn next_arg(&mut self) -> Option<&str> {
        self.next()
    }
}

impl<'a> Iterator for Split<'a> {
    type Item = &'a str;

    fn next(&mut self) -> Option<Self::Item> {
        self.string = self.string.trim_start();

        if self.string.is_empty() {
            return None;
        }

        let (end_char, from) = match self.string.get(0..=0) {
            Some("'") => ('\'', 1),
            Some("\"") => ('"', 1),
            _ => (' ', 0),
        };

        match self.string[from..].find(end_char) {
            Some(end_idx) => {
                let end_idx = end_idx + from;
                let result = &self.string[from..end_idx];
                self.string = &self.string[end_idx+1..];
                Some(result.trim_start())
            },
            None => {
                let result = &self.string[from..];
                self.string = "";
                Some(result.trim())
            }
        }
    }
}
